import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'package:yui/component/basic/global.dart';
import 'package:yui/component/basic/theme.dart';

/// 弹出框
class YDialog {
  static show(Widget dialog,
      {num width: 100,
      num height: 600,
      bool overflowScroll: true,
      bool needHeader: false,
      String title = ''}) {
    if (Yui.context != null) {
      showDialog(
          context: Yui.context,
          barrierDismissible: true,
          builder: (context) {
            return getDialogWarp(
              dialog,
              width: width,
              height: height,
              overflowScroll: overflowScroll,
              needHeader: needHeader,
              title: title,
            );
          });
    } else {
      print('Yui.context必须初始化');
    }
  }

  static getDialogWarp(content,
      {num width: 100,
      num height: 600,
      bool overflowScroll: true,
      bool needHeader: false,
      String title = ''}) {
    final DialogTheme dialogTheme = DialogTheme.of(Yui.context);
    Duration insetAnimationDuration = const Duration(milliseconds: 100);
    Curve insetAnimationCurve = Curves.decelerate;
    print(MediaQuery.of(Yui.context).viewInsets);
    return AnimatedPadding(
      padding: MediaQuery.of(Yui.context).viewInsets +
          EdgeInsets.symmetric(horizontal: Yui.rpx(40), vertical: Yui.rpx(24)),
      duration: insetAnimationDuration,
      curve: insetAnimationCurve,
      child: MediaQuery.removeViewInsets(
        removeLeft: true,
        removeTop: true,
        removeRight: true,
        removeBottom: true,
        context: Yui.context,
        //所以我们其实只需要修改child这个属性了，改成我们想要展示的widget就行了。
        child: Center(
          child: Material(
            type: MaterialType.card,
            shape: dialogTheme.shape,
            child: ConstrainedBox(
              // 设置弹出框的最大的宽高
              constraints: BoxConstraints(
                  minWidth: Yui.rpx(280), maxHeight: Yui.rpx(height)),
              child: Column(
                mainAxisAlignment: MainAxisAlignment.start,
                children: <Widget>[
                  needHeader
                      ? Container(
                          height: Yui.rpx(100),
                          width: double.maxFinite,
                          child: Stack(
                            children: <Widget>[
                              Positioned(
                                left: Yui.rpx(50),
                                top: Yui.rpx(34),
                                child: Text(
                                  title,
                                  style: TextStyle(
                                    fontSize: Yui.rpx(32),
                                  ),
                                ),
                              ),
                              Positioned(
                                right: Yui.rpx(20),
                                top: Yui.rpx(8),
                                child: IconButton(
                                  iconSize: Yui.rpx(36),
                                  icon: Icon(Icons.close),
                                  onPressed: () {
                                    Navigator.pop(Yui.context);
                                  },
                                ),
                              )
                            ],
                          ),
                        )
                      : Container(),
                  Expanded(
                    flex: 1,
                    child: content,
                  )
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }

  static void confirm(
      {String title, String content, Function ok, Function cancel}) {
    showCupertinoDialog(
        context: Yui.context,
        builder: (c) {
          return new CupertinoAlertDialog(
            title: new Text(title),
            content: _getContentWidget(content),
            actions: <Widget>[
              new FlatButton(
                child: new Text('取消'),
                onPressed: () {
                  if (cancel != null) {
                    cancel();
                  }
                  Navigator.of(Yui.context).pop();
                },
              ),
              new FlatButton(
                child: new Text('确认'),
                textColor: YuiColors.primaryColor,
                onPressed: () {
                  if (ok != null) {
                    ok();
                  }
                  Navigator.of(Yui.context).pop();
                },
              ),
            ],
          );
        });
  }

  static void alert({
    String title,
    String content,
    Function ok,
  }) {
    showCupertinoDialog(
        context: Yui.context,
        builder: (c) {
          return new CupertinoAlertDialog(
            title: new Text(title),
            content: _getContentWidget(content),
            actions: <Widget>[
              new FlatButton(
                child: new Text('确认'),
                textColor: YuiColors.primaryColor,
                onPressed: () {
                  if (ok != null) {
                    ok();
                  }
                  Navigator.of(Yui.context).pop();
                },
              ),
            ],
          );
        });
  }

  static Widget _getContentWidget(String content) {
    if (content == null) {
      return Container();
    } else {
      if (content.length < 100) {
        return Text(content);
      } else {
        return Container(
          height: Yui.rpx(600),
          width: double.maxFinite,
          child: CustomScrollView(
            // shrinkWrap: true,
            // 内容
            slivers: <Widget>[
              new SliverPadding(
                padding: EdgeInsets.all(Yui.rpx(20)),
                sliver: new SliverList(
                  delegate: new SliverChildListDelegate(
                    <Widget>[
                      Text(content),
                    ],
                  ),
                ),
              ),
            ],
          ),
        );
      }
    }
  }
}
