import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'package:yui/component/basic/global.dart';
import 'package:yui/component/basic/theme.dart';

typedef Widget PickerItemBuilder(item);

class PickerItemDetails {
  PickerItemDetails({this.item, this.index});
  final dynamic item;
  final int index;

  @override
  String toString() {
    return 'PickerItemDetails{text: $item, index: $index}';
  }
}

class YDatePicker {
  static show() {
    showModalBottomSheet(
        context: Yui.context,
        builder: (_) {
          return Container(
            height: Yui.rpx(500),
            child: CupertinoDatePicker(
              onDateTimeChanged: (dateTime) {},
              initialDateTime: DateTime.now(),
              mode: CupertinoDatePickerMode.date,
            ),
          );
        });
  }
}

class YPicker<T> extends StatefulWidget {
  /// 设置可选择的数据
  List<List> data;

  /// 可以异步获取数据
  Future<List<List>> dataSync;


  /// 可以异步获取数据
  Future<List> childBuilder;

  /// 值, 格式是[value1, value2, value3], 对应数据源的相应级层value
  List<int> value;

  /// 点击选中时执行的回调
  final ValueChanged<PickerItemDetails> onOk;

  /// 点击取消时执行的回调
  final Function onDismiss;

  /// 选中后的回调
  final ValueChanged<PickerItemDetails> onChange;

  /// 每列数据选择变化后的回调函数
  final ValueChanged<PickerItemDetails> onPickerChange;

  /// 选中的文案
  String okText;

  /// 取消选中的文案
  String dismissText;

  /// 大标题
  String title;

  /// 是否可用
  bool disabled;

  /// 是否联动 联动的话会查找节点下面的child 元素
  bool cascade;

  PickerItemBuilder itemBuilder;

  double height;

  YPicker({
    Key key,
    this.data,
    this.dataSync,
    this.value,
    this.cascade = false,
    this.disabled = false,
    this.dismissText = "取消",
    this.okText = "确定",
    this.onChange,
    this.onDismiss,
    this.onOk,
    this.title,
    this.itemBuilder,
    this.onPickerChange,
  })  : assert((data != null && data.length > 0) || dataSync != null),
        super(key: key);

  @override
  PickerState createState() => PickerState();

  /// 展示picker组件
  void show() {
    showModalBottomSheet(
      context: Yui.context,
      builder: (_) {
        return this;
      },
    );
  }
}

class PickerState extends State<YPicker> {
  List oldValue = [];
  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: widget.height != null ? widget.height : Yui.rpx(650),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: <Widget>[
          getHeader(),
          Expanded(
              flex: 1,
              child: Row(
                children: getPickerItem(),
              ))
        ],
      ),
    );
  }

  void initState() {
    super.initState();
    this.oldValue = widget.value;
    if (widget.dataSync != null) {
      widget.dataSync.then<void>((data) {
        setState(() {
          widget.data = data;
          detaData();
        });
      }, onError: (Object error) {
        setState(() {
          widget.data = [];
          detaData();
        });
      });
    } else {
      detaData();
    }
  }

  List<Widget> getPickerItem() {
    List<List> pickerData = widget.data ?? [];
    List<Widget> widgets = [];
    for (var i = 0; i < pickerData.length; i++) {
      if (widget.value.length <= i) {
        widget.value.insert(i, 0);
      }
      widgets.add(Expanded(
        flex: 1,
        child: PickerItem(
          itemBuilder: widget.itemBuilder,
          data: pickerData[i],
          lineHeight: Yui.rpx(100),
          pickerState: this,
          index: i,
          initialItem: widget.value[i],
        ),
      ));
    }
    return widgets;
  }

  /// 处理数据，对有孩子节点的数据进行处理
  detaData([index, child]) {
    if (!widget.cascade) return;
    // List<int> oldVal = widget.value.getRange(0, widget.value.length);
    index ??= 0;
    if (index == 0) {
      child = widget.data[0];
    }
    if (widget.value.length <= index) {
      widget.value.add(0);
    }
    if (child == null) {
      /// 如果指定索引没有元素，则添加，如果有元素 则替换
      widget.data.removeRange(index, widget.data.length);
      widget.value.removeRange(index, widget.value.length);
    } else {
      dynamic data = child[widget.value[index]];
      if (index > 0) {
        /// 如果指定索引没有元素，则添加，如果有元素 则替换
        widget.data.removeRange(index, widget.data.length);
        // widget.value.replaceRange(index, index + 1, [0]);
        if (child != null) {
          widget.data.add(child);
          // widget.value.add(0);
        }
      }
      try {
        if (data != null && data['child'] != null) {
          detaData(index + 1, data['child']);
        }
      } catch (e) {}
    }
  }

  setValue(index, int value) {
    if (widget.value.length <= index) {
      widget.value.add(value);
    } else {
      widget.value.replaceRange(index, index + 1, [value]);
    }
  }

  /// 添加一级选着器
  replacePickerItem(index, data) {
    if (!widget.cascade) return;

    /// 如果指定索引没有元素，则添加，如果有元素 则替换
    setState(() {
      detaData(index, data);
    });
  }

  /// 获取选择器头部显示内容
  Widget getHeader() {
    return Container(
      padding: Yui.edgeInsetsSymmetric(horizontal: 28),
      height: Yui.rpx(100),
      decoration: BoxDecoration(
        color: Colors.white,
      ),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: <Widget>[
          CupertinoButton(
            child: Text(
              "取消",
              style: TextStyle(
                fontSize: Yui.rpx(36),
                color: YuiColors.fontColorblack,
              ),
            ),
            padding: const EdgeInsets.all(0),
            minSize: Yui.rpx(36),
            onPressed: () {
              if (widget.onDismiss != null)
                widget.onDismiss(PickerItemDetails());
              Navigator.of(context).pop();
            },
          ),
          Expanded(
            flex: 1,
            child: Center(
              child: Text(
                widget.title ?? '',
                style: TextStyle(
                  fontSize: Yui.rpx(36),
                  color: YuiColors.fontColorblack,
                ),
              ),
            ),
          ),
          CupertinoButton(
              child: Text(
                "确定",
                style: TextStyle(
                  fontSize: Yui.rpx(36),
                  color: YuiColors.primaryColor,
                ),
              ),
              minSize: Yui.rpx(36),
              padding: const EdgeInsets.all(0),
              onPressed: () {
                if (widget.onOk != null) widget.onOk(PickerItemDetails());
                Navigator.of(context).pop();
              }),
        ],
      ),
    );
  }
}

class PickerItem extends StatefulWidget {
  /// 初始化位置
  int initialItem;

  /// 可选择的数据
  List data;

  PickerItemBuilder itemBuilder;

  /// 当前picker 的引用
  PickerState pickerState;

  /// 改Item所在的位置
  int index;

  /// 每列元素高度
  double lineHeight;
  PickerItem({
    Key key,
    this.data,
    this.index,
    this.lineHeight,
    this.itemBuilder,
    this.pickerState,
    this.initialItem,
  }) : super(key: key);

  _PickerItemState createState() => _PickerItemState();
}

class _PickerItemState extends State<PickerItem> {
  FixedExtentScrollController scrollController;
  PickerItemDetails itemDetails;

  void initState() {
    super.initState();
    scrollController =
        FixedExtentScrollController(initialItem: widget.initialItem);
    itemDetails = PickerItemDetails(
      index: widget.initialItem,
      item: widget.data.elementAt(widget.initialItem),
    );
  }

  /// 处理级联的孩子元素
  dealChild(int index, stateChange) {
    dynamic data = widget.data[index];
    if (hasChild(data)) {
      widget.pickerState.replacePickerItem(widget.index + 1, data['child']);
    } else {
      widget.pickerState.replacePickerItem(widget.index + 1, null);
    }
  }

  hasChild(data) {
    bool flag = false;
    try {
      if (data != null && data['child'] != null) {
        flag = true;
      }
    } catch (e) {}
    return flag;
  }

  @override
  Widget build(BuildContext context) {
    return CupertinoPicker(
      backgroundColor: Colors.white,
      itemExtent: widget.lineHeight,
      scrollController: scrollController,
      onSelectedItemChanged: (index) async {
        itemDetails =
            PickerItemDetails(index: index, item: widget.data.elementAt(index));
        if (widget.pickerState.widget.onChange != null) {
          widget.pickerState.widget.onChange(itemDetails);
          widget.pickerState.widget.onPickerChange(itemDetails);
        }
        widget.pickerState.setValue(widget.index, index);
        await Future.delayed(Duration(milliseconds: 500), () {
          dealChild(index, true);
        });
      },
      children: widget.data.map((item) {
        return SizedBox(
          height: widget.lineHeight,
          child: widget.itemBuilder != null
              ? widget.itemBuilder(item)
              : Center(
                  child: Text(
                  item,
                  style: TextStyle(
                      fontSize: Yui.rpx(40), color: YuiColors.fontColorblack),
                )),
        );
      }).toList(),
    );
  }

  @override
  void dispose() {
    scrollController?.dispose();
    super.dispose();
  }
}
