import 'package:flutter/material.dart';
import 'dart:async';

import 'package:yui/component/basic/global.dart';

class YToast {
  static _ToastState _toast;
  static void init(_ToastState toast) {
    _toast = toast;
  }

  static show(@required String text, {Widget icon, int duration = 2000}) {
    _toast.show(text: text, icon: icon, duration: duration);
  }

  static hide() {}
}

class Toast extends StatefulWidget {
  _ToastState createState() => _ToastState();
}

class _ToastState extends State<Toast> with SingleTickerProviderStateMixin {
  Timer timer;
  bool isShow = false;
  String _text = '';
  Widget _icon;
  Animation<double> animation;
  AnimationController controller;
  @override
  void initState() {
    controller = new AnimationController(
        duration: const Duration(milliseconds: 100), vsync: this);
    animation = new Tween(begin: 0.3, end: 1.0).animate(controller);
    YToast.init(this);
  }

  @override
  Widget build(BuildContext context) {
    if (!isShow) return Container();
    return Positioned(
      child: new Directionality(
        textDirection: TextDirection.ltr,
        child: Center(
          child: ScaleTransition(
            scale: animation,
            child: getContent(),
          ),
        ),
      ),
    );
  }

  Widget getContent() {
    if (_icon != null) {
      return Container(
        width: Yui.rpx(360),
        height: Yui.rpx(360),
        padding: EdgeInsets.symmetric(vertical: 10, horizontal: 10),
        decoration: BoxDecoration(
            color: Color.fromRGBO(0, 0, 0, 0.5),
            borderRadius: BorderRadius.all(Radius.circular(4))),
        child: Column(
          children: <Widget>[
            Container(
              padding: EdgeInsets.all(Yui.rpx(10)),
              width: Yui.rpx(250),
              height: Yui.rpx(250),
              child: _icon,
            ),
            Text(
              _text,
              maxLines: 2,
              overflow: TextOverflow.ellipsis,
              style: TextStyle(color: Colors.white, fontSize: Yui.rpx(24)),
            )
          ],
        ),
      );
    } else {
      return Container(
        constraints: BoxConstraints(
          maxWidth: Yui.rpx(600),
        ),
        padding: EdgeInsets.symmetric(vertical: 6, horizontal: 10),
        decoration: BoxDecoration(
            color: Color.fromRGBO(0, 0, 0, 0.5),
            borderRadius: BorderRadius.all(Radius.circular(4))),
        child: Text(
          _text,
          maxLines: 2,
          overflow: TextOverflow.ellipsis,
          style: TextStyle(color: Colors.white, fontSize: Yui.rpx(24)),
        ),
      );
    }
  }

  show({String text, Widget icon, int duration = 2000}) {
    setState(() {
      isShow = true;
      _text = text;
      _icon = icon;
      controller.forward();
    });
    if (timer != null) {
      timer.cancel();
    }
    timer = new Timer(Duration(milliseconds: duration), () {
      setState(() {
        isShow = false;
        _text = '';
        _icon = null;
        controller.reset();
      });
    });
  }

  @override
  void dispose() {
    controller.dispose();
    timer.cancel();
    super.dispose();
  }
}
