import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';

class YCell extends StatelessWidget {
  /// 在标题前面的一个组件.
  ///
  /// 例如 [Icon] 或者 [CircleAvatar] 组件.
  final Widget leading;

  /// 主要的内容标题.
  final Widget title;

  /// 次级标题，用来描述主标题
  ///
  /// 如果 [isThreeLine] 为 false, 这不会被换行.
  ///
  /// 如果 [isThreeLine] 为 true, 将会变成最多2行显示
  final Widget subtitle;

  /// 最末端的一个标题显示，可用用来放箭头等
  final Widget trailing;

  /// subtitle 是否可以2行显示
  final bool isThreeLine;

  /// tile 的内边距.
  final EdgeInsetsGeometry contentPadding;

  /// 是否可点击.
  final bool enabled;

  /// 点击后调用
  final GestureTapCallback onTap;

  /// 长按调用
  final GestureLongPressCallback onLongPress;

  /// 是否被选中
  final bool selected;

  /// 顶部分割线
  bool topDivider;

  /// 底部分割线
  bool bottomDivider;

  YCell({
    this.topDivider = false,
    this.bottomDivider = true,
    this.leading,
    this.title,
    this.subtitle,
    this.trailing,
    this.isThreeLine = false,
    this.contentPadding,
    this.enabled = true,
    this.onTap,
    this.onLongPress,
    this.selected = false,
  });
  @override
  Widget build(BuildContext context) {
    return Column(
      children: <Widget>[
        Visibility(
          visible: topDivider,
          child: Divider(
            height: 0,
          ),
        ),
        ListTile(
          leading: leading,
          title: title,
          subtitle: subtitle,
          trailing: trailing,
          isThreeLine: isThreeLine,
          contentPadding: contentPadding,
          enabled: enabled,
          onTap: onTap,
          onLongPress: onLongPress,
          selected: selected,
        ),
        Visibility(
          visible: bottomDivider,
          child: Divider(
            height: 0,
          ),
        ),
      ],
    );
  }
}
