import 'package:flutter/material.dart';
import 'dart:async';

class YNoticebar extends StatefulWidget{

  final String text;
  final TextStyle textStyle;
  ///滚动方向，水平或者垂直
  final Axis scrollAxis;
  ///空白部分占控件的百分比(前一个距离后面一个的距离))
  final double ratioOfBlankToScreen;
   // 单位时间内移动的距离
  final double moveDistance;

  YNoticebar({
    @required this.text,
    this.textStyle,
    this.scrollAxis:Axis.horizontal,
    this.ratioOfBlankToScreen:0.25,
    this.moveDistance:0.3,
  }) :assert(text!=null,);

  @override
  State<StatefulWidget> createState() {
    return   YNoticebarState();
  }
}

class YNoticebarState extends State<YNoticebar> with SingleTickerProviderStateMixin{

  ScrollController scroController;
  double screenWidth;
  double screenHeight;
  double position=0.0;
  Timer timer;
  // 单位时间内移动的距离
  // final double _moveDistance= 0.3;
  // 移动速度   值越大  越慢
  final int _timerRest=100;
  GlobalKey _key=GlobalKey();


  @override
  void initState() {
    super.initState();
    scroController= ScrollController();
    WidgetsBinding.instance.addPostFrameCallback((callback){
      startTimer();
    });
  }

  void startTimer(){
    double widgetWidth = _key.currentContext.findRenderObject().paintBounds.size.width;
    double widgetHeight = _key.currentContext.findRenderObject().paintBounds.size.height;

    timer=Timer.periodic( Duration(milliseconds: _timerRest), (timer){
      double maxScrollExtent=scroController.position.maxScrollExtent;
      double pixels=scroController.position.pixels;
      if(pixels+widget.moveDistance>=maxScrollExtent){
        if(widget.scrollAxis==Axis.horizontal){
          //TODO 我也看不懂怎么算的
          position=(maxScrollExtent-screenWidth*widget.ratioOfBlankToScreen+widgetWidth)/2-widgetWidth+pixels-maxScrollExtent;
        }else{
          position=(maxScrollExtent-screenHeight*widget.ratioOfBlankToScreen+widgetHeight)/2-widgetHeight+pixels-maxScrollExtent;
        }
        print(position);
        scroController.jumpTo(position);
      }
      position+=widget.moveDistance;
      scroController.animateTo(position,duration:  Duration(milliseconds: _timerRest),curve: Curves.linear);
    });
  }


  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    screenWidth=MediaQuery.of(context).size.width;
    screenHeight=MediaQuery.of(context).size.height;
  }

  Widget getBothEndsChild(){
    if(widget.scrollAxis ==Axis.vertical){
      String newString=widget.text.split("").join("\n");
      return  Center(
        child:  Text(newString,style: widget.textStyle,textAlign: TextAlign.center,),
      );
    }
    return  Center(
        child: Text(widget.text,style: widget.textStyle)
      
    );
  }

  Widget getCenterChild(){
    if(widget.scrollAxis ==Axis.horizontal){
      return  Container(width: screenWidth*widget.ratioOfBlankToScreen);
    }else{
      return  Container(height: screenHeight*widget.ratioOfBlankToScreen);
    }
  }



  @override
  void dispose() {
    super.dispose();
    timer.cancel();
  }

  @override
  Widget build(BuildContext context) {
    return   ListView(
      key: _key,
      scrollDirection: widget.scrollAxis,
      controller: scroController,
      physics:  NeverScrollableScrollPhysics(),
      children: <Widget>[
        getBothEndsChild(),
        getCenterChild(),
        getBothEndsChild(),
      ],
    );
  }
}