import 'dart:async';
import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';
import 'package:yui/component/feedback/picker.dart';
import 'package:yui/component/feedback/toast.dart';
import 'package:yui/component/layout/cell.dart';
import 'package:yui/main.dart';

class PickerDemo extends ExampleBase {
  PickerDemo(String path) : super(path);

  Future<List<List>> getCity() async {
    String config = await rootBundle.loadString("assets/area.json");
    return [json.decode(config)['china']['child']];
  }

  @override
  Widget getWidget(BuildContext context) {
    return ListView(
      children: <Widget>[
        Container(
            margin: EdgeInsets.symmetric(vertical: 20),
            decoration: BoxDecoration(
              color: Colors.white,
            ),
            child: Column(
              children: <Widget>[
                YCell(
                  title: Text(
                    '选择器',
                  ),
                  subtitle: Text('基础选择器'),
                  onTap: () {
                    YPicker(
                      data: [
                        ['item1-1', 'item2-1', 'item3-1']
                      ],
                      value: [1],
                    ).show();
                  },
                ),
                YCell(
                  title: Text(
                    '多列选择',
                  ),
                  subtitle: Text('多列选择器'),
                  onTap: () {
                    YPicker(
                      onDismiss: (PickerItemDetails info) {
                        YToast.show('onDismiss ${json.encode(info.item)}');
                      },
                      onOk: (PickerItemDetails info) {
                        YToast.show('onOk ${json.encode(info.item)}');
                      },
                      onPickerChange: (PickerItemDetails info) {
                        YToast.show('onPickerChange ${json.encode(info.item)}');
                      },
                      onChange: (PickerItemDetails info) {
                        YToast.show('change ${json.encode(info.item)}');
                      },
                      data: [
                        ['item1-1', 'item2-1', 'item3-1'],
                        ['item1-2', 'item2-2'],
                        ['item1-2', 'item2-2'],
                      ],
                      value: [1, 1, 0],
                    ).show();
                  },
                ),
                YCell(
                  title: Text(
                    '选择器',
                  ),
                  subtitle: Text('级联选择，并自定义每一列的内容'),
                  onTap: () {
                    YPicker(
                      data: [
                        [
                          {
                            'name': 'stone1',
                            "child": [
                              {
                                'name': 'stone1',
                                "child": [
                                  {'name': 'stone1-1'},
                                  {'name': 'stone1-2'},
                                ]
                              },
                              {'name': 'stone2'},
                            ]
                          },
                          {
                            'name': 'stone2',
                            "child": [
                              {'name': 'stone2-1'},
                              {'name': 'stone2-2'},
                            ]
                          },
                          {
                            'name': 'stone3',
                            "child": [
                              {
                                'name': 'stone1-3',
                                "child": [
                                  {'name': 'stone2-3'},
                                  {'name': 'stone3-4'},
                                ]
                              },
                              {
                                'name': 'stone4',
                                "child": [
                                  {'name': 'stone4-3'},
                                  {'name': 'stone4-4'},
                                ]
                              },
                            ]
                          },
                        ]
                      ],
                      value: [2, 1, 1],
                      cascade: true,
                      itemBuilder: (item) {
                        return Center(
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: <Widget>[
                              Icon(Icons.ac_unit),
                              Text(
                                item['name'],
                                style: TextStyle(fontSize: 17),
                              )
                            ],
                          ),
                        );
                      },
                    ).show();
                  },
                ),
                YCell(
                  title: Text(
                    '城市选择',
                  ),
                  subtitle: Text('异步获取出具数据，级联显示'),
                  onTap: () {
                    YPicker(
                      dataSync: getCity(),
                      value: [3, 2, 1],
                      cascade: true,
                      title: '城市选择',
                      itemBuilder: (item) {
                        return Center(
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: <Widget>[
                              Text(
                                item['name'],
                                style: TextStyle(fontSize: 17),
                              )
                            ],
                          ),
                        );
                      },
                    ).show();
                  },
                ),
                YCell(
                  title: Text(
                    '日期选择',
                  ),
                  onTap: () {
                    YDatePicker.show();
                  },
                ),
              ],
            )),
      ],
    );
  }
}
