import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:yui/component/basic/button.dart';
import 'package:yui/component/basic/global.dart';
import 'package:yui/component/basic/theme.dart';
import 'package:yui/component/layout/cell.dart';
import 'package:yui/main.dart';

class FormDemo extends ExampleBase {
  FormDemo(String path) : super(path);

  @override
  Widget getWidget(BuildContext context) {
    return Container(
      padding: EdgeInsets.symmetric(vertical: 10, horizontal: 10),
      child: _HomePage(),
    );
  }
}

class _HomePage extends StatefulWidget {
  @override
  _HomePageState createState() => new _HomePageState();
}

class _HomePageState extends State<_HomePage> {
  GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  String _name = "王";
  String _password;
  String remark;
  String city = "";
  String phone = "15923222222";
  int sex = 0;
  bool hasMoney = true;
  bool sendMessage = false;

  void _forSubmitted() {
    var _form = _formKey.currentState;

    if (_form.validate()) {
      _form.save();
      print(_name);
      print(_password);
    } else {
      print('验证失败');
    }
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      decoration: BoxDecoration(
        color: Colors.white,
      ),
      child: new Form(
        key: _formKey,
        child: ListTileTheme(
          style: ListTileStyle.drawer,
          textColor: YuiColors.fontColorblack,
          contentPadding: EdgeInsets.symmetric(vertical: 1, horizontal: 10),
          child: new ListView(
            children: <Widget>[
              /// 直接使用FormField
              YCell(
                title: FormField(
                  builder: (FormFieldState field) {
                    return TextField(
                      keyboardType: TextInputType.text, //获取焦点时,启用的键盘类型
                      style: new TextStyle(
                        color: Colors.green,
                      ), // 设置字体样式
                      textAlign: TextAlign.right, //输入的内容在水平方向如何显示
                      decoration: new InputDecoration(
                        icon: Text('城市'),
                        border:
                            OutlineInputBorder(borderSide: BorderSide.none), //
                        hintText: '请选择你要投保的城市',
                      ),
                      onTap: () {
                        showDialog(
                            context: context,
                            builder: (context) {
                              return Container(
                                constraints: BoxConstraints(maxHeight: 300),
                                child: CupertinoPicker(
                                  backgroundColor: Colors.white,
                                  itemExtent: 50,
                                  onSelectedItemChanged: (int index) {},
                                  children: <Widget>[
                                    Text('1',style: TextStyle(fontSize: Yui.rpx(36)),),
                                    Text('2',style: TextStyle(fontSize: Yui.rpx(36)),),
                                    Text('3',style: TextStyle(fontSize: Yui.rpx(36)),),
                                    Text('4',style: TextStyle(fontSize: Yui.rpx(36)),),
                                    Text('5',style: TextStyle(fontSize: Yui.rpx(36)),),
                                    Text('6',style: TextStyle(fontSize: Yui.rpx(36)),),
                                  ],
                                ),
                              );
                            });
                        print('选中');
                      },
                      onChanged: (val) {
                        print('change' + val);
                      },
                      onSubmitted: (String v) {
                        print('submit' + v);
                      },
                    );
                  },
                ),
              ),
              // 定义验证
              YCell(
                title: FormField<String>(
                  initialValue: phone,
                  validator: (String value) {
                    if (value != '110') {
                      return '你电话必须是110';
                    }
                  },
                  builder: (FormFieldState<String> field) {
                    return Column(
                      children: <Widget>[
                        TextField(
                            controller:
                                TextEditingController(text: field.value),
                            textAlign: TextAlign.right, //输入的内容在水平方向如何显示
                            decoration: InputDecoration(
                              icon: Text('电话'),
                              hintText: '电话号码',
                              border: OutlineInputBorder(
                                borderSide: BorderSide.none,
                              ), //
                              errorText: field.errorText,
                            ),
                            onChanged: (String val) {
                              field.setValue(val);
                            }),
                      ],
                    );
                  },
                ),
              ),
              YCell(
                title: TextFormField(
                  initialValue: _name,
                  decoration: new InputDecoration(
                    icon: Text('姓名'),
                    border:
                        OutlineInputBorder(borderSide: BorderSide.none), // 边框样式
                  ),
                  onSaved: (val) {
                    _name = val;
                  },
                ),
              ),
              // 复杂类型表单使用 formField 包裹。使用YCell布局
              FormField(
                builder: (FormFieldState field) {
                  return YCell(
                    title: Text('是否有钱'),
                    onTap: () {
                      setState(() {
                        hasMoney = !hasMoney;
                      });
                    },
                    trailing: Checkbox(
                      value: hasMoney,
                      onChanged: (bool value) {
                        setState(() {
                          hasMoney = value;
                        });
                      },
                    ),
                  );
                },
              ),
              YCell(
                title: DropdownButtonFormField<int>(
                  value: sex,
                  decoration: new InputDecoration(
                    icon: Text('性别 下拉选择实现1'),
                    border: OutlineInputBorder(borderSide: BorderSide.none),
                  ),
                  onChanged: (int value) {
                    setState(() {
                      sex = value;
                    });
                  },
                  items: [
                    DropdownMenuItem<int>(
                      value: 1,
                      child: Text('男'),
                    ),
                    DropdownMenuItem<int>(
                      value: 0,
                      child: Text('女'),
                    )
                  ],
                ),
              ),
              FormField(
                builder: (FormFieldState field) {
                  return YCell(
                    title: Text('性别 下拉选择实现2'),
                    trailing: DropdownButton(
                      value: sex,
                      onChanged: (int value) {
                        setState(() {
                          sex = value;
                        });
                      },
                      items: [
                        DropdownMenuItem<int>(
                          value: 1,
                          child: Text('男'),
                        ),
                        DropdownMenuItem<int>(
                          value: 0,
                          child: Text('女'),
                        )
                      ],
                    ),
                  );
                },
              ),
              FormField(
                builder: (FormFieldState field) {
                  return YCell(
                    title: Text('是否有钱'),
                    onTap: () {
                      setState(() {
                        hasMoney = !hasMoney;
                      });
                    },
                    trailing: CupertinoSwitch(
                      value: hasMoney,
                      activeColor: YuiColors.primaryColor,
                      onChanged: (bool value) {
                        setState(() {
                          hasMoney = value;
                        });
                      },
                    ),
                  );
                },
              ),
              YCell(
                title: TextFormField(
                  decoration: new InputDecoration(
                    icon: Text('密码'),
                    border: OutlineInputBorder(borderSide: BorderSide.none),
                  ),

                  /// 隐藏文字内容
                  obscureText: true,
                  validator: (val) {
                    return val.length < 4 ? "密码长度错误" : null;
                  },
                  onSaved: (val) {
                    _password = val;
                  },
                ),
              ),
              YCell(
                title: TextFormField(
                  decoration: new InputDecoration(
                    icon: Text('备注'),
                    border: OutlineInputBorder(borderSide: BorderSide.none),
                  ),
                  maxLines: 2,
                  maxLength: 300,
                  onSaved: (val) {
                    remark = val;
                  },
                ),
              ),
              YButton(
                title: '提交',
                onPressed: _forSubmitted,
              )
            ],
          ),
        ),
      ),
    );
  }
}
