import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:yui/component/basic/app_warp.dart';
import 'package:yui/component/basic/global.dart';
import 'package:yui/component/basic/theme.dart';
import 'package:yui/example/_common/base.dart';
import 'package:yui/example/_common/component_items.dart';
import 'package:yui/example/_common/config.dart';
import 'package:yui/example/_common/screenutil.dart';
import 'dart:convert';

List<GroupItem> groupList = [];

class ItemLabel {
  String title;
  String docPath;
  String desc;
  ExampleBase page;
  ItemLabel(this.title, this.desc, this.docPath, this.page);
}

/// 定义每个分组的数据信息
class GroupItem {
  IconData iconData;
  String group;
  String title;
  String label;
  List<ItemLabel> items = [];
  GroupItem({this.iconData, this.group, this.title, this.label, this.items});
}

List<GroupItem> jsonToGroupList(dynamic options) {
  List<GroupItem> list = [];
  (options as List).forEach((dynamic value) {
    GroupItem gitem = GroupItem(
      group: value['group'],
      title: value['title'],
      label: value['label'],
      items: [],
    );
    (value['items'] as List).forEach((dynamic item) {
      String componentKey = item['component'];
      String path = value['ename'] + '/' + item['ename'];
      if (allMap[componentKey] != null) {
        gitem.items.add(ItemLabel(item['name'], item['desc'] ?? "", path,
            allMap[componentKey](path)));
      } else {
        print('你需要在 lib/example/_common/config.dart 里面配置新加的组件 ${componentKey}');
      }
    });
    list.add(gitem);
  });
  return list;
}

/// 定义每个组件测试用例都需要实现的基础类
class ExampleBase {
  String path;
  ExampleBase(this.path);

  Widget getWidget(BuildContext context) {
    return Container();
  }

  Widget item({BuildContext context, String title, List<Widget> children}) {
    List<Widget> list = <Widget>[];
    list.add(
      DemoTitle(title),
    );
    list.add(SizedBox(
      height: Yui.rpx(20),
    ));
    list.addAll(children);
    return Container(
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: list,
      ),
    );
  }
}

ThemeData themeData = ThemeData(
  highlightColor: YuiColors.highlightColor,
  primaryColor: YuiColors.primaryColor,
  buttonTheme: ButtonThemeData(
    buttonColor: YuiColors.primaryColor,
    disabledColor: YuiColors.disabledColor,
    highlightColor: YuiColors.highlightColor,
    splashColor: YuiColors.primaryColor,
  ),
  tabBarTheme: TabBarTheme(
    
  ),
  toggleableActiveColor: YuiColors.primaryColor,
  appBarTheme: AppBarTheme(
    elevation: 0,
    color: Colors.white,
    iconTheme: IconThemeData(
      color: YuiColors.fontColorblack1,
    ),
    textTheme: TextTheme(
      title: TextStyle(
        color: YuiColors.fontColorblack,
        fontSize: 20,
      ),
      body1: TextStyle(
        color: YuiColors.fontColorblack1,
      ),
    ),
  ),
);

// 本地调试入口
void main() async {
  String config = await rootBundle.loadString("assets/config.json");
  groupList = jsonToGroupList(json.decode(config));
  runApp(AppWarp(FlutterView()));
}

class FlutterView extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: '',
      theme: themeData,
      home: Scaffold(
        body: GroupListWidget(),
      ),
    );
  }
}

class GroupListWidget extends StatelessWidget {
  final Widget child;
  GroupListWidget({Key key, this.child}) : super(key: key);
  @override
  Widget build(BuildContext context) {
    Yui.init(context);
    return CustomScrollView(slivers: <Widget>[
      SliverAppBar(
        brightness: Brightness.light,
        pinned: true,
        expandedHeight: 100.0,
        backgroundColor: YuiColors.primaryColor,
        flexibleSpace: FlexibleSpaceBar(
          title: Text(
            'YUI',
            style: TextStyle(color: Colors.white),
          ),
          background: Image.asset(
            'assets/top.jpg',
            fit: BoxFit.cover,
          ),
        ),
      ),
      new SliverList(
          delegate: new SliverChildListDelegate(
              groupList.map<Widget>((GroupItem value) {
        return ItemIcon(
          leftChild: Icon(
            Icons.settings_input_component,
            color: YuiColors.primaryColor,
          ),
          title: '${value.title} (${value.items.length})',
          label: value.label,
          onTap: (c) => _onTap(c, value.title, value.items),
        );
      }).toList()))
    ]);
  }

  _onTap(BuildContext context, String title, List<ItemLabel> list) {
    Navigator.of(context).push(MaterialPageRoute(
        builder: (c) => ComponentItemPage(title, list ?? [])));
  }
}

class ItemIcon extends StatelessWidget {
  final Widget leftChild;
  final String title;
  final String label;
  final Function onTap;
  ItemIcon(
      {Key key,
      @required this.leftChild,
      @required this.title,
      @required this.onTap,
      this.label = ''})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      decoration: BoxDecoration(
          boxShadow: [
            BoxShadow(color: Colors.grey[200], blurRadius: 20, spreadRadius: 12)
          ],
          color: Colors.white,
          borderRadius: BorderRadius.all(Radius.circular(4))),
      margin: EdgeInsets.fromLTRB(10, 14, 10, 0),
      child: Material(
        // elevation: 2,
        child: InkWell(
          child: Container(
              padding: EdgeInsets.symmetric(vertical: 20, horizontal: 20),
              child: Row(
                children: <Widget>[
                  leftChild,
                  SizedBox(
                    width: 20,
                  ),
                  Expanded(
                      child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      Text(
                        title,
                        style: TextStyle(fontSize: 18),
                      ),
                      Text(label, style: TextStyle(color: Colors.grey))
                    ],
                  )),
                  Icon(
                    Icons.chevron_right,
                    color: YuiColors.primaryColor,
                  )
                ],
              )),
          onTap: () {
            // print('object');
            this.onTap(context);
          },
        ),
      ),
    );
  }
}
